@@ -1,4 +1,4 @@
-# $Id: Cart.pm 776 2005-09-08 23:07:29Z claco $
+# $Id: Cart.pm 1134 2006-05-16 22:04:20Z claco $
 package Catalyst::Helper::Model::Handel::Cart;
 use strict;
 use warnings;
@@ -26,12 +26,9 @@ __model__
 package [% class %];
 use strict;
 use warnings;
-use base 'Handel::Cart';
+use base qw/Handel::Cart/;
 
-{
-    $^W = 0;
-    Handel::DBI->connection('[% dsn %]', '[% user %]', '[% pass %]');
-};
+__PACKAGE__->connection_info('[% dsn %]', '[% user %]', '[% pass %]');
 
 1;
 __test__
@@ -75,4 +72,4 @@ L<Catalyst::Manual>, L<Catalyst::Helper>, L<Handel::Cart>
     Christopher H. Laco
     CPAN ID: CLACO
     claco@chrislaco.com
-    http://today.icantfocus.com/blog/
\ No newline at end of file
+    http://today.icantfocus.com/blog/
