@@ -1,41 +1,45 @@
-# $Id: Cart.pm 598 2005-07-29 01:43:05Z claco $
+# $Id: Cart.pm 1202 2006-06-04 18:59:10Z claco $
 package Handel::Cart;
 use strict;
 use warnings;
 
 BEGIN {
-    use base 'Handel::DBI';
-    use Handel::Constants qw(:cart :returnas);
-    use Handel::Constraints qw(:all);
+    use Handel;
+    use Handel::Constants qw/:cart :returnas/;
+    use Handel::Constraints qw/:all/;
     use Handel::Currency;
-    use Handel::L10N qw(translate);
+    use Handel::L10N qw/translate/;
+
+    use base qw/Handel::Storage/;
+    __PACKAGE__->schema_class('Handel::Cart::Schema');
+    __PACKAGE__->schema_source('Carts');
+    __PACKAGE__->item_class('Handel::Cart::Item');
+    __PACKAGE__->setup_column_accessors;
+
+    __PACKAGE__->add_constraint('id',      id      => \&constraint_uuid);
+    __PACKAGE__->add_constraint('shopper', shopper => \&constraint_uuid);
+    __PACKAGE__->add_constraint('type',    type    => \&constraint_cart_type);
+    __PACKAGE__->add_constraint('name',    name    => \&constraint_cart_name);
+
+    __PACKAGE__->default_values({
+        id   => \&Handel::Storage::uuid,
+        type => CART_TYPE_TEMP
+    });
 };
 
-__PACKAGE__->autoupdate(1);
-__PACKAGE__->table('cart');
-__PACKAGE__->iterator_class('Handel::Iterator');
-__PACKAGE__->columns(All => qw(id shopper type name description));
-__PACKAGE__->has_many(_items => 'Handel::Cart::Item', 'cart');
-__PACKAGE__->add_constraint('id',      id      => \&constraint_uuid);
-__PACKAGE__->add_constraint('shopper', shopper => \&constraint_uuid);
-__PACKAGE__->add_constraint('type',    type    => \&constraint_cart_type);
-
 sub new {
-    my ($self, $data) = @_;
+    my ($class, $data) = @_;
+    $class = ref $class || $class;
 
     throw Handel::Exception::Argument(
         -details => translate('Param 1 is not a HASH reference') . '.') unless
             ref($data) eq 'HASH';
 
-    if (!defined($data->{'id'}) || !constraint_uuid($data->{'id'})) {
-        $data->{'id'} = $self->uuid;
-    };
+    my $self = bless {
+        storage => $class->schema_instance->resultset($class->schema_source)->create($data)
+    }, $class;
 
-    if (!defined($data->{'type'})) {
-        $data->{'type'} = CART_TYPE_TEMP;
-    };
-
-    return $self->create($data);
+    return $self;
 };
 
 sub add {
@@ -47,29 +51,27 @@ sub add {
               (ref($data) eq 'HASH' or $data->isa('Handel::Cart::Item'));
 
     if (ref($data) eq 'HASH') {
-        if (!defined($data->{'id'}) || !constraint_uuid($data->{'id'})) {
-            $data->{'id'} = $self->uuid;
-        };
-
-        return $self->add_to__items($data);
+        return bless {
+            storage => $self->storage->create_related($self->item_relationship, $data)
+        }, $self->item_class;
     } else {
         my %copy;
 
-        foreach ($data->columns) {
+        foreach ($data->storage->columns) {
             next if $_ =~ /^(id|cart)$/i;
-            $copy{$_} = $data->$_;
+            $copy{$_} = $data->storage->$_;
         };
 
-        $copy{'id'} = $self->uuid;
-
-        return $self->add_to__items(\%copy);
+        return bless {
+            storage => $self->storage->create_related($self->item_relationship, \%copy)
+        }, $self->item_class;
     };
 };
 
 sub clear {
     my $self = shift;
 
-    $self->_items->delete_all;
+    $self->storage->delete_related($self->item_relationship);
 
     return undef;
 };
@@ -77,7 +79,7 @@ sub clear {
 sub count {
     my $self  = shift;
 
-    return $self->_items->count || 0;
+    return $self->storage->count_related($self->item_relationship) || 0;
 };
 
 sub delete {
@@ -87,81 +89,69 @@ sub delete {
         translate('Param 1 is not a HASH reference') . '.') unless
             ref($filter) eq 'HASH';
 
-    ## I'd much rather use $self->_items->search_like, but it doesn't work that
-    ## way yet. This should be fine as long as :weaken refs works.
-    return Handel::Cart::Item->search_like(%{$filter},
-        cart => $self->id)->delete_all;
+    $filter = $self->_migrate_wildcards($filter);
+
+    return $self->storage->delete_related($self->item_relationship, $filter);
+};
+
+sub destroy {
+    my ($self, $filter) = @_;
+
+    if (ref $self) {
+        $self->storage->delete;
+    } else {
+        throw Handel::Exception::Argument( -details =>
+            translate('Param 1 is not a HASH reference') . '.') unless
+                ref($filter) eq 'HASH';
+
+        $filter = $self->_migrate_wildcards($filter);
+
+        $self->schema_instance->resultset($self->schema_source)->search($filter)->delete_all;
+    };
+
+    return;
 };
 
 sub items {
-    my ($self, $filter, $wantiterator) = @_;
+    my ($self, $filter) = @_;
 
     throw Handel::Exception::Argument( -details =>
         translate('Param 1 is not a HASH reference') . '.') unless(
             ref($filter) eq 'HASH' or !$filter);
 
-    $wantiterator ||= RETURNAS_AUTO;
-    $filter       ||= {};
+    $filter = $self->_migrate_wildcards($filter);
 
-    my $wildcard = Handel::DBI::has_wildcard($filter);
+    if (wantarray) {
+        my @items = $self->storage->search_related($self->item_relationship, $filter)->all;
 
-    ## If the filter as a wildcard, push it through a fresh search_like since it
-    ## doesn't appear to be available within a loaded object.
-    if ((wantarray && $wantiterator != RETURNAS_ITERATOR) || $wantiterator == RETURNAS_LIST) {
-        my @items = $wildcard ?
-            Handel::Cart::Item->search_like(%{$filter}, cart => $self->id) :
-            $self->_items(%{$filter});
-
-        return @items;
-    } elsif ($wantiterator == RETURNAS_ITERATOR) {
-        my $iterator = $wildcard ?
-            Handel::Cart::Item->search_like(%{$filter}, cart => $self->id) :
-            $self->_items(%{$filter});
+        return map {bless {storage => $_}, $self->item_class} @items;
+    } else {
+        my $iterator = $self->storage->search_related_rs($self->item_relationship, $filter);
+        $iterator->result_class($self->item_class);
 
         return $iterator;
-    } else {
-        my $iterator = $wildcard ?
-            Handel::Cart::Item->search_like(%{$filter}, cart => $self->id) :
-            $self->_items(%{$filter});
-        if ($iterator->count == 1 && $wantiterator != RETURNAS_ITERATOR && $wantiterator != RETURNAS_LIST) {
-            return $iterator->next;
-        } else {
-            return $iterator;
-        };
     };
 };
 
 sub load {
-    my ($self, $filter, $wantiterator) = @_;
+    my ($class, $filter) = @_;
+    $class = ref $class || $class;
 
     throw Handel::Exception::Argument( -details =>
         translate('Param 1 is not a HASH reference') . '.') unless(
             ref($filter) eq 'HASH' or !$filter);
 
-    $wantiterator ||= RETURNAS_AUTO;
+    $filter = $class->_migrate_wildcards($filter);
 
-    ## only return array if wantarray and not explicitly asking for an iterator
-    ## or we've explicitly asked for a list/array
-    if ((wantarray && $wantiterator != RETURNAS_ITERATOR) || $wantiterator == RETURNAS_LIST) {
-        my @carts = $filter ? $self->search_like(%{$filter}) :
-            $self->retrieve_all;
-        return @carts;
-    ## return an iterator if explicitly asked for
-    } elsif ($wantiterator == RETURNAS_ITERATOR) {
-        my $iterator = $filter ?
-            $self->search_like(%{$filter}) : $self->retrieve_all;
+    if (wantarray) {
+        my @carts = $class->schema_instance->resultset($class->schema_source)->search($filter)->all;
+
+        return map {bless {storage => $_}, $class} @carts;
+    } else {
+        my $iterator = $class->schema_instance->resultset($class->schema_source)->search_rs($filter);
+        $iterator->result_class($class);
 
         return $iterator;
-    ## full out auto
-    } else {
-        my $iterator = $filter ?
-            $self->search_like(%{$filter}) : $self->retrieve_all;
-
-        if ($iterator->count == 1 && $wantiterator != RETURNAS_ITERATOR && $wantiterator != RETURNAS_LIST) {
-            return $iterator->next;
-        } else {
-            return $iterator;
-        };
     };
 };
 
@@ -175,32 +165,32 @@ sub restore {
             'Param 1 is not a HASH reference or Handel::Cart') . '.') unless(
                 ref($data) eq 'HASH' or $data->isa('Handel::Cart'));
 
+    if (ref $data eq 'HASH') {
+        $data = $self->_migrate_wildcards($data);
+    };
+
     my @carts = (ref($data) eq 'HASH') ?
-        Handel::Cart->search_like(%{$data}) : $data;
+        $self->load($data)->all : $data;
 
     if ($mode == CART_MODE_REPLACE) {
         $self->clear;
 
         my $first = $carts[0];
-        $self->autoupdate(0);
         $self->name($first->name);
         $self->description($first->description);
-        $self->update;
-        $self->autoupdate(1);
 
         foreach (@carts) {
-            my $iterator = $_->items(undef, 1);
-            while (my $item = $iterator->next) {
+            my @items = $_->items->all;
+            foreach my $item (@items) {
                 $self->add($item);
             };
         };
     } elsif ($mode == CART_MODE_MERGE) {
         foreach (@carts) {
-            my $iterator = $_->items(undef, 1);
-            while (my $item = $iterator->next) {
-                if (my $exists = $self->items({sku => $item->sku})){
+            my @items = $_->items->all;
+            foreach my $item (@items) {
+                if (my $exists = $self->items({sku => $item->sku})->first){
                     $exists->quantity($item->quantity + $exists->quantity);
-                    $exists->update;
                 } else {
                     $self->add($item);
                 };
@@ -208,8 +198,8 @@ sub restore {
         };
     } elsif ($mode == CART_MODE_APPEND) {
         foreach (@carts) {
-            my $iterator = $_->items(undef, 1);
-            while (my $item = $iterator->next) {
+            my @items = $_->items->all;
+            foreach my $item (@items) {
                 $self->add($item);
             };
         };
@@ -220,18 +210,19 @@ sub restore {
 };
 
 sub save {
-    $_[0]->type(CART_TYPE_SAVED);
+    my $self = shift;
+    $self->type(CART_TYPE_SAVED);
 
     return undef;
 };
 
 sub subtotal {
     my $self     = shift;
-    my $it       = $self->items(undef, 1);
+    my $it       = $self->items();
     my $subtotal = 0.00;
 
-    while ( my $item = $it->next ) {
-        $subtotal += ( $item->total );
+    while (my $item = $it->next) {
+        $subtotal += ($item->total);
     };
 
     return Handel::Currency->new($subtotal);
@@ -321,6 +312,21 @@ not a hashref.
 
 =head1 METHODS
 
+=head2 item_class($classname)
+
+Sets the name of the class to be used when returning or creating cart items.
+While you can set this directly in your application, it's best to set it
+in a custom subclass of Handel::Cart.
+
+    package CustomCart;
+    use strict;
+    use warnings;
+    use base 'Handel::Cart';
+
+    __PACKAGE__->item_class('CustomCart::CustomItem';
+
+    1;
+
 =head2 Adding Cart Items
 
 You can add items to the shopping cart by supplying a hashref containing the
@@ -428,6 +434,23 @@ returns the number of items deleted.
 
 =back
 
+=head2 Removing the Entire Cart
+
+=over
+
+=item C<$cart-E<gt>destroy(\%filter)>
+
+=item C<Handel::Cart-E<gt>destroy(\%filter)>
+
+When called used as an instance method, this will delete all items from the
+current cart instance and delete the cart container. C<filter> will be ignored.
+
+When called as a package method, this will delete all carts matching C<filter>.
+A Handel::Exception::Argument exception will be thrown is C<filter> is not a
+HASH reference.
+
+=back
+
 =head2 Saving Your Cart
 
 By default every shopping cart created is considered temporary
