@@ -1,10 +1,10 @@
-# $Id: ConfigReader.pm 727 2005-08-19 02:32:16Z claco $
+# $Id: ConfigReader.pm 1111 2006-04-12 23:44:49Z claco $
 package Handel::ConfigReader;
 use strict;
 use warnings;
-use vars qw(%Defaults);
-use Tie::Hash;
-use base 'Tie::StdHash';
+use vars qw(%Defaults $MOD_PERL);
+
+my $instance;
 
 %Defaults = (
     HandelMaxQuantityAction => 'Adjust',
@@ -12,6 +12,26 @@ use base 'Tie::StdHash';
     HandelCurrencyFormat    => 'FMT_STANDARD'
 );
 
+BEGIN {
+    use Tie::Hash;
+    use base 'Tie::StdHash';
+
+    if (exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERSION} == 2) {
+        require Apache2::RequestRec;
+        require Apache2::RequestUtil;
+        require Apache2::RequestIO;
+        require Apache2::ServerUtil;
+
+        $MOD_PERL = 2;
+    } elsif ($ENV{MOD_PERL}) {
+        require Apache;
+
+        $MOD_PERL = 1;
+    } else {
+        $MOD_PERL = 0;
+    };
+};
+
 sub new {
     my $class = shift;
     my %config;
@@ -20,6 +40,12 @@ sub new {
     return bless \%config, $class;
 };
 
+sub instance {
+    $instance ||= shift->new(@_);
+
+    return $instance;
+};
+
 sub get {
     my ($self, $key) = (shift, shift);
     my $default = shift || $Defaults{$key} || '';
@@ -32,17 +58,33 @@ sub FETCH {
     my $default = $Defaults{$key} || '';
     my $value   = '';
 
-    if ($ENV{MOD_PERL}) {
-        require Apache;
-        my $r = Apache->request;
+    if ($MOD_PERL == 2) {
+        my $c = eval {
+            Apache2::RequestUtil->request || Apache2::ServerUtil->server;
+        };
 
-        $value = $r->dir_config($key) || $ENV{$key} || $default;
+        if ($c) {
+            $value = eval{$c->dir_config($key)} || $ENV{$key} || $default;
+        };
+    } elsif ($MOD_PERL == 1) {
+        my $c = eval {
+            Apache->request || Apache->server;
+        };
+
+        if ($c) {
+            $value = $c->dir_config($key) || $ENV{$key} || $default;
+        };
     };
 
     if (!$value) {
         $value = $ENV{$key} || $default;
     };
 
+    # quick untaint for now. Assuming we'll mever get a ref from system os ENV
+    if (! ref $value && $value =~ /^(.*)$/g) {
+        $value = $1;
+    };
+
     return $value;
 };
 
@@ -84,17 +126,23 @@ the same:
     my $setting = $cfg->get('Setting');
     my $setting = $cfg->{'Setting'};
 
-Thie latter is the preferred usage in anticipation of als integrating
+The latter is the preferred usage in anticipation of also integrating
 Apache::ModuleConfig and custom directives which use the same hash syntax.
 
 =head1 CONSTRUCTOR
 
 =head2 new
 
-Returns a new Handel::ConfigReader object.
+Returns a new Handel::ConfigReader instance.
 
     my $cfg = Handel::ConfigReader->new();
 
+=head2 instance
+
+Returns the singleton Handel::ConfigReader instance.
+
+    my $cfg = Handel::ConfigReader->instance();
+
 =head1 METHODS
 
 =head2 get($key [, $default])
@@ -130,7 +178,7 @@ C<HandelMaxQuantityAction> below.
 =head2 HandelMaxQuantityAction (Adjust|Exception)
 
 This option defines what action should be taken when a cart items quantity is being set
-to something above C<HandelMaxQuantity>. When set to C<Adjust> the quantity qill simple
+to something above C<HandelMaxQuantity>. When set to C<Adjust> the quantity will simply
 be reset to C<HandelMaxQuantity> and no exception will be raised. This is the default
 action.
 
@@ -174,6 +222,12 @@ The user name used to connect to the server. Defaults to C<commerce>.
 
 The password used to connect to the server. Defaults to C<commerce>.
 
+=head2 HandelDBIDSN
+
+The full data source to the connect to the database. If a dsn is supplied
+the driver/host/port and name are ignored. IF no dsn is supplied, one will
+will be constructed from driver/host/port and name.
+
 =head2 HandelPluginPaths
 
 This resets the checkout plugin search path to a namespace of your choosing,
@@ -185,7 +239,7 @@ In the example above, the checkout plugin search path will load all plugins
 in the MyApp::Plugins::* namespace (but not MyApp::Plugin itself). Any plugins
 in Handel::Checkout::Plugin::* will be ignored.
 
-You can also pass a comma or space seperate list of namespaces.
+You can also pass a comma or space separate list of namespaces.
 
     PerlSetVar HandelPluginPaths 'MyApp::Plugins, OtherApp::Plugins'
 
@@ -195,7 +249,7 @@ will be ignored.
 =head2 HandelAddPluginPaths
 
 This adds an additional plugin search paths. This can be a comma or space
-seperated list of namespaces.
+separated list of namespaces.
 
     PerlSetVar HandelAddPluginPaths  'MyApp::Plugins, OtherApp::Plugins'
 
@@ -208,7 +262,7 @@ will be ignored.
 
 =head2 HandelIgnorePlugins
 
-This is a comma/space seperated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to ignore when loading
+This is a comma/space separated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to ignore when loading
 all available plugins in the given namespaces.
 
     PerlSetVar HandelIgnorePlugins 'Handel::Checkout::Plugin::Initialize'
@@ -233,7 +287,7 @@ they will be ignored.
 
 =head2 HandelLoadPlugins
 
-This is a comma or space seperated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to be loaded from the available namespaces.
+This is a comma or space separated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to be loaded from the available namespaces.
 
     PerlSetVar HandelLoadPlugins 'Handel::Checkout::Plugin::ValidateAddress'
 
