@@ -1,4 +1,4 @@
-# $Id: Constraints.pm 697 2005-08-10 01:29:51Z claco $
+# $Id: Constraints.pm 1198 2006-06-03 02:32:37Z claco $
 package Handel::Constraints;
 use strict;
 use warnings;
@@ -16,6 +16,7 @@ BEGIN {
                 &constraint_price
                 &constraint_uuid
                 &constraint_cart_type
+                &constraint_cart_name
                 &constraint_currency_code
                 &constraint_checkout_phase
                 &constraint_order_type
@@ -29,7 +30,7 @@ sub constraint_quantity {
     my $value = defined $_[0] ? shift : '';
     my ($object, $column, $changing) = @_;
 
-    my $cfg    = $Handel::Cfg;
+    my $cfg    = Handel->config;
     my $max    = $cfg->{'HandelMaxQuantity'};
     my $action = $cfg->{'HandelMaxQuantityAction'};
 
@@ -49,7 +50,7 @@ sub constraint_quantity {
 sub constraint_price {
     my $value = defined $_[0] ? shift : '';
 
-    return ($value =~ /^\d{1,5}(\.\d{1,2})?$/ && $value > 0);
+    return ($value =~ /^\d{1,5}(\.\d{1,2})?$/);
 };
 
 sub constraint_uuid {
@@ -74,6 +75,17 @@ sub constraint_cart_type {
     return 1;
 };
 
+sub constraint_cart_name {
+    my ($value, $object, $column, $changing) = @_;
+    my $type = ref $changing ? $changing->{'type'} : '';
+
+    if (constraint_cart_type($type) && $type == CART_TYPE_SAVED && !length($value || '')) {
+        return 0;
+    };
+    return 1;
+};
+
+
 sub constraint_currency_code {
     my $value = defined $_[0] ? uc(shift) : '';
 
@@ -95,11 +107,13 @@ sub constraint_checkout_phase {
 
     return if $value !~ /[0-9]/;
 
-    if ($value != CHECKOUT_PHASE_INITIALIZE && $value != CHECKOUT_PHASE_VALIDATE &&
-        $value != CHECKOUT_PHASE_AUTHORIZE && $value != CHECKOUT_PHASE_DELIVER) {
-        return 0;
+    foreach my $const (@{Handel::Constants->CHECKOUT_ALL_PHASES}) {
+        if ($value == $const) {
+            return 1;
+        };
     };
-    return 1;
+
+    return 0;
 };
 
 sub constraint_order_type {
@@ -189,6 +203,11 @@ otherwise it returns C<undef>.
 Returns 1 if the value passed is C<ORDER_TYPE_SAVED> or C<ORDER_TYPE_TEMP>,
 otherwise it returns C<undef>.
 
+=head2 constraint_cart_name
+
+Returns 0 if the cart type is C<CART_TYPE_SAVED> and the name is undefined,
+otherwise it returns 1.
+
 =head1 EXPORT_TAGS
 
 =head2 :all
