@@ -1,12 +1,11 @@
-# $Id: Currency.pm 478 2005-03-22 00:50:54Z claco $
+# $Id: Currency.pm 1145 2006-05-17 01:21:34Z claco $
 package Handel::Currency;
 use strict;
 use warnings;
 use overload '""' => \&stringify, fallback => 1;
-use Handel::ConfigReader;
 
 BEGIN {
-    use Handel::ConfigReader;
+    use Handel;
     use Handel::Constraints qw(:all);
     use Handel::Exception;
     use Handel::L10N qw(translate);
@@ -30,7 +29,7 @@ sub format {
     eval 'use Locale::Currency::Format';
     return $self->{'price'} if $@;
 
-    my $cfg = Handel::ConfigReader->new;
+    my $cfg = Handel->config;
 
     eval '$format = ' .  ($format || $cfg->{'HandelCurrencyFormat'});
     $code   ||= $cfg->{'HandelCurrencyCode'};
@@ -44,7 +43,7 @@ sub format {
 
 sub convert {
     my ($self, $from, $to, $format, $options) = @_;
-    my $cfg = Handel::ConfigReader->new;
+    my $cfg = Handel->config;
 
     $from ||= $cfg->{'HandelCurrencyCode'};
     $to   ||= $cfg->{'HandelCurrencyCode'};
@@ -94,7 +93,7 @@ __END__
 
 =head1 NAME
 
-Handel::Currency - Price container to do currency formatting
+Handel::Currency - Price container to do currency conversion/formatting
 
 =head1 SYNOPSIS
 
@@ -111,7 +110,7 @@ Handel::Currency - Price container to do currency formatting
 =head1 DESCRIPTION
 
 The C<Handel::Currency> module provides basic currency formatting within
-Handel. It can be used seperately to format any number into a more friendly
+Handel. It can be used separately to format any number into a more friendly
 format:
 
     my $price = 1.23;
@@ -135,7 +134,7 @@ based price.
 =head2 new
 
 The create a new Handel::Currency instance, simply call C<new> and pass in the
-price to be formated:
+price to be formatted:
 
     my $currency = Handel::Currency->new(10.23);
 
@@ -143,10 +142,10 @@ price to be formated:
 
 =head2 format([$code, $options])
 
-The C<format> method returns the freshly formated price in a currency and
+The C<format> method returns the freshly formatted price in a currency and
 format declared in C<Locale::Currency::Format>. If no currency code or
 format are specified, the defaults values from C<Handel::ConfigReader> are
-used. Currencly those defaults are C<USD> and C<FMT_STANDARD>.
+used. Currently those defaults are C<USD> and C<FMT_STANDARD>.
 
 It is also acceptable to specify different default values.
 See L</"CONFIGURATION"> and C<Handel::ConfigReader> for further details.
