@@ -1,4 +1,4 @@
-# $Id: Exception.pm 495 2005-05-30 01:38:24Z claco $
+# $Id: Exception.pm 1150 2006-05-19 00:21:13Z claco $
 package Handel::Exception;
 use strict;
 use warnings;
@@ -6,6 +6,10 @@ use warnings;
 BEGIN {
     use base 'Error';
     use Handel::L10N qw(translate);
+    eval 'require Apache::AxKit::Exception';
+    if (!$@) {
+        push @__PACKAGE__::ISA, 'Apache::AxKit::Exception';
+    };
 };
 
 my $lh = Handel::L10N->get_handle();
@@ -44,7 +48,6 @@ sub new {
         -text => 'The supplied field(s) failed database constraints', @_ );
 };
 
-
 package Handel::Exception::Argument;
 use strict;
 use warnings;
@@ -137,7 +140,7 @@ unexpected things happen.
 =head2 Handel::Exception
 
 This is the base exception thrown in C<Handel>. All other exceptions subclass
-C<Handel::Exception> so it's possibl to catch all Handel generated exceptions
+C<Handel::Exception> so it's possible to catch all Handel generated exceptions
 with a single C<catch> statement.
 
     try {
