@@ -1,84 +1,71 @@
-# $Id: Order.pm 793 2005-09-12 02:05:09Z claco $
+# $Id: Order.pm 1202 2006-06-04 18:59:10Z claco $
 package Handel::Order;
 use strict;
 use warnings;
 
 BEGIN {
-    use base 'Handel::DBI';
-    use Handel::Cart;
+    use Handel;
     use Handel::Checkout;
-    use Handel::Constants qw(:checkout :returnas :order);
-    use Handel::Constraints qw(:all);
+    use Handel::Constants qw/:checkout :returnas :order/;
+    use Handel::Constraints qw/:all/;
     use Handel::Currency;
-    use Handel::L10N qw(translate);
+    use Handel::L10N qw/translate/;
+    use Scalar::Util qw/blessed/;
+
+    use base qw/Handel::Storage/;
+    __PACKAGE__->schema_class('Handel::Order::Schema');
+    __PACKAGE__->schema_source('Orders');
+    __PACKAGE__->item_class('Handel::Order::Item');
+    __PACKAGE__->cart_class('Handel::Cart');
+    __PACKAGE__->setup_column_accessors;
+
+    __PACKAGE__->add_constraint('id',       id       => \&constraint_uuid);
+    __PACKAGE__->add_constraint('shopper',  shopper  => \&constraint_uuid);
+    __PACKAGE__->add_constraint('type',     type     => \&constraint_order_type);
+    __PACKAGE__->add_constraint('shipping', shipping => \&constraint_price);
+    __PACKAGE__->add_constraint('handling', handling => \&constraint_price);
+    __PACKAGE__->add_constraint('subtotal', subtotal => \&constraint_price);
+    __PACKAGE__->add_constraint('tax',      tax      => \&constraint_price);
+    __PACKAGE__->add_constraint('total',    total    => \&constraint_price);
+
+    __PACKAGE__->mk_classaccessors(qw/ccn cctype ccm ccy ccvn ccname ccissuenumber ccstartdate ccenddate/);
+
+    __PACKAGE__->default_values({
+        id       => \&Handel::Storage::uuid,
+        type     => ORDER_TYPE_TEMP,
+        shipping => 0,
+        handling => 0,
+        subtotal => 0,
+        tax      => 0,
+        total    => 0
+    });
 };
 
-__PACKAGE__->autoupdate(1);
-__PACKAGE__->table('orders');
-__PACKAGE__->iterator_class('Handel::Iterator');
-__PACKAGE__->columns(All => qw(id shopper type number created updated comments
-    shipmethod shipping handling tax subtotal total
-    billtofirstname billtolastname billtoaddress1 billtoaddress2 billtoaddress3
-    billtocity billtostate billtozip billtocountry  billtodayphone
-    billtonightphone billtofax billtoemail shiptosameasbillto
-    shiptofirstname shiptolastname shiptoaddress1 shiptoaddress2 shiptoaddress3
-    shiptocity shiptostate shiptozip shiptocountry shiptodayphone
-    shiptonightphone shiptofax shiptoemail)
-);
-__PACKAGE__->columns(
-    TEMP => qw(ccn cctype ccm ccy ccvn ccname)
-);
-
-
-__PACKAGE__->has_many(_items => 'Handel::Order::Item', 'orderid');
-__PACKAGE__->has_a(subtotal  => 'Handel::Currency');
-__PACKAGE__->has_a(total     => 'Handel::Currency');
-__PACKAGE__->has_a(shipping  => 'Handel::Currency');
-__PACKAGE__->has_a(handling  => 'Handel::Currency');
-__PACKAGE__->has_a(tax       => 'Handel::Currency');
-
-__PACKAGE__->add_constraint('id',       id       => \&constraint_uuid);
-__PACKAGE__->add_constraint('shopper',  shopper  => \&constraint_uuid);
-__PACKAGE__->add_constraint('type',     type     => \&constraint_order_type);
-__PACKAGE__->add_constraint('shipping', shipping => \&constraint_price);
-__PACKAGE__->add_constraint('handling', handling => \&constraint_price);
-__PACKAGE__->add_constraint('subtotal', subtotal => \&constraint_price);
-__PACKAGE__->add_constraint('tax',      tax      => \&constraint_price);
-__PACKAGE__->add_constraint('total',    total    => \&constraint_price);
-
 sub new {
-    my ($self, $data, $process) = @_;
+    my ($class, $data, $process) = @_;
+    $class = ref $class || $class;
 
     throw Handel::Exception::Argument(
         -details => translate('Param 1 is not a HASH reference') . '.') unless
             ref($data) eq 'HASH';
 
-    if (!defined($data->{'id'}) || !constraint_uuid($data->{'id'})) {
-        $data->{'id'} = $self->uuid;
-    };
-
-    if (!defined($data->{'type'})) {
-        $data->{'type'} = ORDER_TYPE_TEMP;
-    };
-
-    my $cart = $data->{'cart'};
+    my $cart = delete $data->{'cart'};
     my $is_uuid = constraint_uuid($cart);
-    delete $data->{'cart'};
 
     if (defined $cart) {
         throw Handel::Exception::Argument( -details =>
           translate(
               'Cart reference is not a HASH reference or Handel::Cart') . '.') unless
-                  (ref($cart) eq 'HASH' or UNIVERSAL::isa($cart, 'Handel::Cart') or $is_uuid);
+                  (ref($cart) eq 'HASH' or (blessed($cart) && $cart->isa('Handel::Cart')) or $is_uuid);
 
         if (ref $cart eq 'HASH') {
-            $cart = Handel::Cart->load($cart, RETURNAS_ITERATOR)->first;
+            $cart = $class->cart_class->load($cart)->first;
 
             throw Handel::Exception::Order( -details =>
                 translate(
                     'Could not find a cart matching the supplied search criteria') . '.') unless $cart;
         } elsif ($is_uuid) {
-            $cart = Handel::Cart->load({id => $cart}, RETURNAS_ITERATOR)->first;
+            $cart = $class->cart_class->load({id => $cart})->first;
 
             throw Handel::Exception::Order( -details =>
                 translate(
@@ -91,32 +78,18 @@ sub new {
                     $cart->count > 0;
     };
 
-    my $order = $self->create($data);
+    if (defined $cart) {
+        $data->{'shopper'} = $cart->shopper unless $data->{'shopper'};
+    };
+
+    my $order = bless {
+        storage => $class->schema_instance->resultset($class->schema_source)->create($data)
+    }, $class;
+
 
     if (defined $cart) {
-        my $subtotal = 0;
-        my $items = $cart->items(undef, RETURNAS_ITERATOR);
-        if ($cart->shopper && !$order->shopper) {
-            $order->shopper($cart->shopper);
-        };
-        while (my $item = $items->next) {
-            my %copy;
-
-            foreach ($item->columns) {
-                next if $_ =~ /^(id|cart)$/i;
-                $copy{$_} = $item->$_;
-            };
-
-            $copy{'id'} = $self->uuid unless constraint_uuid($copy{'id'});
-            $copy{'orderid'} = $order->id;
-            $copy{'total'} = $copy{'quantity'}*$copy{'price'};
-            $subtotal += $copy{'total'};
-
-            $order->add_to__items(\%copy);
-        };
-
-        $order->subtotal($subtotal);
-        $order->update;
+        $class->copy_cart($order, $cart);
+        $class->copy_cart_items($order, $cart);
     };
 
     if ($process) {
@@ -127,7 +100,7 @@ sub new {
         if ($status == CHECKOUT_STATUS_OK) {
             $checkout->order->update;
         } else {
-            $order->SUPER::delete;
+            $order->destroy;
             undef $order;
         };
         undef $checkout;
@@ -136,6 +109,39 @@ sub new {
     return $order;
 };
 
+sub copy_cart {
+    my ($self, $order, $cart) = @_;
+
+    if ($cart->shopper && !$order->shopper) {
+        $order->shopper($cart->shopper);
+    };
+
+    $order->subtotal($cart->subtotal);
+    $order->update;
+};
+
+sub copy_cart_items {
+    my ($self, $order, $cart) = @_;
+    my %columns = map {$_ => $_} $order->schema_class->class($order->item_class->schema_source)->columns;
+
+    foreach my $item ($cart->items) {
+        my %copy;
+
+        foreach ($cart->item_class->schema_class->class($cart->item_class->schema_source)->columns) {
+            next if $_ =~ /^(id|cart)$/i;
+            next unless (exists $columns{$_});
+
+            $copy{$_} = $item->storage->$_;
+        };
+
+        $copy{'id'} = $self->uuid unless Handel::Constraints::constraint_uuid($copy{'id'});
+        $copy{'orderid'} = $order->id;
+        $copy{'total'} = $copy{'quantity'}*$copy{'price'};
+
+        $order->storage->create_related($self->item_relationship, \%copy);
+    };
+};
+
 sub add {
     my ($self, $data) = @_;
 
@@ -145,32 +151,32 @@ sub add {
               (ref($data) eq 'HASH' or $data->isa('Handel::Order::Item') or $data->isa('Handel::Cart::Item'));
 
     if (ref($data) eq 'HASH') {
-        if (!defined($data->{'id'}) || !constraint_uuid($data->{'id'})) {
-            $data->{'id'} = $self->uuid;
-        };
-
-        return $self->add_to__items($data);
+        return bless {
+            storage => $self->storage->create_related($self->item_relationship, $data)
+        }, $self->item_class;
     } else {
         my %copy;
 
-        foreach ($data->columns) {
+        foreach ($data->storage->columns) {
             next if $_ =~ /^(id|orderid|cart)$/i;
-            $copy{$_} = $data->$_;
+            $copy{$_} = $data->storage->$_;
         };
-        if (UNIVERSAL::isa($data, 'Handel::Cart::Item')) {
+        if (blessed($data) && $data->isa('Handel::Cart::Item')) {
             $copy{'total'} = $data->total;
         };
 
         $copy{'id'} = $self->uuid;
 
-        return $self->add_to__items(\%copy);
+        return bless {
+            storage => $self->storage->create_related($self->item_relationship, \%copy)
+        }, $self->item_class;
     };
 };
 
 sub clear {
     my $self = shift;
 
-    $self->_items->delete_all;
+    $self->storage->delete_related($self->item_relationship);
 
     return undef;
 };
@@ -178,7 +184,7 @@ sub clear {
 sub count {
     my $self  = shift;
 
-    return $self->_items->count || 0;
+    return $self->storage->count_related($self->item_relationship) || 0;
 };
 
 sub delete {
@@ -188,81 +194,69 @@ sub delete {
         translate('Param 1 is not a HASH reference') . '.') unless
             ref($filter) eq 'HASH';
 
-    ## I'd much rather use $self->_items->search_like, but it doesn't work that
-    ## way yet. This should be fine as long as :weaken refs works.
-    return Handel::Order::Item->search_like(%{$filter},
-        orderid => $self->id)->delete_all;
+    $filter = $self->_migrate_wildcards($filter);
+
+    return $self->storage->delete_related($self->item_relationship, $filter);
+};
+
+sub destroy {
+    my ($self, $filter) = @_;
+
+    if (ref $self) {
+        $self->storage->delete;
+    } else {
+        throw Handel::Exception::Argument( -details =>
+            translate('Param 1 is not a HASH reference') . '.') unless
+                ref($filter) eq 'HASH';
+
+        $filter = $self->_migrate_wildcards($filter);
+
+        $self->schema_instance->resultset($self->schema_source)->search($filter)->delete_all;
+    };
+
+    return;
 };
 
 sub items {
-    my ($self, $filter, $wantiterator) = @_;
+    my ($self, $filter) = @_;
 
     throw Handel::Exception::Argument( -details =>
         translate('Param 1 is not a HASH reference') . '.') unless(
             ref($filter) eq 'HASH' or !$filter);
 
-    $wantiterator ||= RETURNAS_AUTO;
-    $filter       ||= {};
+    $filter = $self->_migrate_wildcards($filter);
 
-    my $wildcard = Handel::DBI::has_wildcard($filter);
+    if (wantarray) {
+        my @items = $self->storage->search_related($self->item_relationship, $filter)->all;
 
-    ## If the filter as a wildcard, push it through a fresh search_like since it
-    ## doesn't appear to be available within a loaded object.
-    if ((wantarray && $wantiterator != RETURNAS_ITERATOR) || $wantiterator == RETURNAS_LIST) {
-        my @items = $wildcard ?
-            Handel::Order::Item->search_like(%{$filter}, orderid => $self->id) :
-            $self->_items(%{$filter});
-
-        return @items;
-    } elsif ($wantiterator == RETURNAS_ITERATOR) {
-        my $iterator = $wildcard ?
-            Handel::Order::Item->search_like(%{$filter}, orderid => $self->id) :
-            $self->_items(%{$filter});
+        return map {bless {storage => $_}, $self->item_class} @items;
+    } else {
+        my $iterator = $self->storage->search_related_rs($self->item_relationship, $filter);
+        $iterator->result_class($self->item_class);
 
         return $iterator;
-    } else {
-        my $iterator = $wildcard ?
-            Handel::Order::Item->search_like(%{$filter}, orderid => $self->id) :
-            $self->_items(%{$filter});
-        if ($iterator->count == 1 && $wantiterator != RETURNAS_ITERATOR && $wantiterator != RETURNAS_LIST) {
-            return $iterator->next;
-        } else {
-            return $iterator;
-        };
     };
 };
 
 sub load {
-    my ($self, $filter, $wantiterator) = @_;
+    my ($class, $filter) = @_;
+    $class = ref $class || $class;
 
     throw Handel::Exception::Argument( -details =>
         translate('Param 1 is not a HASH reference') . '.') unless(
             ref($filter) eq 'HASH' or !$filter);
 
-    $wantiterator ||= RETURNAS_AUTO;
+    $filter = $class->_migrate_wildcards($filter);
 
-    ## only return array if wantarray and not explicitly asking for an iterator
-    ## or we've explicitly asked for a list/array
-    if ((wantarray && $wantiterator != RETURNAS_ITERATOR) || $wantiterator == RETURNAS_LIST) {
-        my @orders = $filter ? $self->search_like(%{$filter}) :
-            $self->retrieve_all;
-        return @orders;
-    ## return an iterator if explicitly asked for
-    } elsif ($wantiterator == RETURNAS_ITERATOR) {
-        my $iterator = $filter ?
-            $self->search_like(%{$filter}) : $self->retrieve_all;
+    if (wantarray) {
+        my @carts = $class->schema_instance->resultset($class->schema_source)->search($filter)->all;
+
+        return map {bless {storage => $_}, $class} @carts;
+    } else {
+        my $iterator = $class->schema_instance->resultset($class->schema_source)->search_rs($filter);
+        $iterator->result_class($class);
 
         return $iterator;
-    ## full out auto
-    } else {
-        my $iterator = $filter ?
-            $self->search_like(%{$filter}) : $self->retrieve_all;
-
-        if ($iterator->count == 1 && $wantiterator != RETURNAS_ITERATOR && $wantiterator != RETURNAS_LIST) {
-            return $iterator->next;
-        } else {
-            return $iterator;
-        };
     };
 };
 
@@ -278,13 +272,13 @@ sub reconcile {
                   (ref($cart) eq 'HASH' or UNIVERSAL::isa($cart, 'Handel::Cart') or $is_uuid);
 
         if (ref $cart eq 'HASH') {
-            $cart = Handel::Cart->load($cart, RETURNAS_ITERATOR)->first;
+            $cart = $self->cart_class->load($cart)->first;
 
             throw Handel::Exception::Order( -details =>
                 translate(
                     'Could not find a cart matching the supplied search criteria') . '.') unless $cart;
         } elsif ($is_uuid) {
-            $cart = Handel::Cart->load({id => $cart}, RETURNAS_ITERATOR)->first;
+            $cart = $self->cart_class->load({id => $cart})->first;
 
             throw Handel::Exception::Order( -details =>
                 translate(
@@ -299,11 +293,8 @@ sub reconcile {
 
     if ($self->subtotal != $cart->subtotal || $self->count != $cart->count) {
         $self->clear;
-        my @citems = $cart->items;
-        foreach my $item (@citems) {
-            $self->add($item);
-        };
-        $self->subtotal($cart->subtotal);
+        $self->copy_cart($self, $cart);
+        $self->copy_cart_items($self, $cart);
     };
 };
 
@@ -456,15 +447,108 @@ This method removes all items from the current cart object.
 
     $cart->clear;
 
+=head2 copy_cart
+
+When creating a new order from an existing cart, C<copy_cart> will be called to
+copy the carts contents into the new order object. If you are using custom cart
+or order subclasses, the default copy_cart will only copy the fields declared in
+Handel::Cart, ignoring any custom fields you may add.
+
+To fix this, simply subclass Handel::Order and override copy_cart. As its
+parameters, it will receive the order and cart objects.
+
+    package CustomOrder;
+    use base 'Handel::Order';
+
+    sub copy_cart {
+        my ($self, $order, $cart) = @_;
+
+        # copy stock fields
+        $self->SUPER::copy_cart($order, $cart);
+
+        # now catch the custom ones
+        $order->customfield($cart->customfield);
+    };
+
+=head2 copy_cart_items
+
+When creating a new order from an existing cart, C<copy_cart_items> will be
+called to copy the cart items into the new order object items. If you are using
+custom cart or order subclasses, the default copy_cart_item will only copy the
+fields declared in Handel::Cart::Item, ignoring any custom fields you may add.
+
+To fix this, simply subclass Handel::Order and override copy_cart. As its
+parameters, it will receive the order and cart objects.
+
+    package CustomOrder;
+    use base 'Handel::Order';
+
+    __PACKAGE__->cart_class('CustomCart');
+
+    sub copy_cart_items {
+        my ($self, $order, $cart) = @_;
+        my $items = $cart->items(undef, RETURNAS_ITERATOR);
+
+        while (my $item = $items->next) {
+            my %copy;
+
+            foreach (CustomCart::Item->columns) {
+                next if $_ =~ /^(id|cart)$/i;
+                $copy{$_} = $item->$_;
+            };
+
+            $copy{'id'} = $self->uuid unless constraint_uuid($copy{'id'});
+            $copy{'orderid'} = $order->id;
+            $copy{'total'} = $copy{'quantity'}*$copy{'price'};
+
+            $order->add_to__items(\%copy);
+        };
+    };
+
 =head2 delete(\%filter)
 
-This method deletes the cart item(s) matching the supplied filter values and
+This method deletes the order item(s) matching the supplied filter values and
 returns the number of items deleted.
 
     if ( $cart->delete({id => '8D4B0BE1-C02E-11D2-A33D-00A0C94B8D0E'}) ) {
         print 'Item deleted';
     };
 
+=head2 destroy(\%filter)
+
+When called used as an instance method, this will delete all items from the
+current cart instance and delete the cart container. C<filter> will be ignored.
+
+When called as a package method, this will delete all carts matching C<filter>.
+A Handel::Exception::Argument exception will be thrown is C<filter> is not a
+HASH reference.
+
+=head2 cart_class($orderclass)
+
+Gets/Sets the name of the class to use when loading existing cart into the
+new order. By default, it loads carts using Handel::Cart. While you can set this
+directly in your application, it's best to set it in a custom subclass of
+Handel::Order.
+
+    package CustomOrder;
+    use base 'Handel::Order';
+    __PACKAGE__->cart_class('CustomCart');
+
+=head2 item_class($classname)
+
+Gets/Sets the name of the class to be used when returning or creating order items.
+While you can set this directly in your application, it's best to set it
+in a custom subclass of Handel::Order.
+
+    package CustomOrder;
+    use strict;
+    use warnings;
+    use base 'Handel::Order';
+
+    __PACKAGE__->item_class('CustomOrder::CustomItem';
+
+    1;
+
 =head2 items([\%filter, [$wantiterator])
 
 You can retrieve all or some of the items contained in the order via the C<items>
@@ -567,6 +651,69 @@ Gets/sets the bill to fax number
 
 Gets/sets the bill to email address
 
+=head2 ccn*
+
+Gets/sets the credit cart number.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 cctype*
+
+Gets/sets the credit cart type.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccm*
+
+Gets/sets the credit cart expiration month.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccy*
+
+Gets/sets the credit cart expiration year.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccvn*
+
+Gets/sets the credit cart verification number.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccname*
+
+Gets/sets the credit cart holders name as it appears on the card.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccissuenumber*
+
+Gets/sets the credit cart issue number.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccstartdate*
+
+Gets/sets the credit cart start date.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
+=head2 ccenddate*
+
+Gets/sets the credit cart end date.
+
+B<NOTE:> This field is stored in memory for the life of the order instance and
+is not a real database field.
+
 =head2 comments
 
 Gets/sets the comments for this order
