@@ -1,40 +1,43 @@
-# $Id: Item.pm 650 2005-08-05 00:45:21Z claco $
+# $Id: Item.pm 1202 2006-06-04 18:59:10Z claco $
 package Handel::Order::Item;
 use strict;
 use warnings;
 
 BEGIN {
-    use base 'Handel::DBI';
+    use Handel;
     use Handel::Constraints qw(:all);
     use Handel::Currency;
     use Handel::L10N qw(translate);
+
+    use base qw/Handel::Storage/;
+    __PACKAGE__->schema_class('Handel::Order::Schema');
+    __PACKAGE__->schema_source('Items');
+    __PACKAGE__->setup_column_accessors;
+    __PACKAGE__->add_constraint('quantity', quantity => \&constraint_quantity);
+    __PACKAGE__->add_constraint('price',    price    => \&constraint_price);
+    __PACKAGE__->add_constraint('id',       id       => \&constraint_uuid);
+    __PACKAGE__->add_constraint('orderid',  orderid  => \&constraint_uuid);
+    __PACKAGE__->add_constraint('total',    total    => \&constraint_price);
+
+    __PACKAGE__->default_values({
+        id    => \&Handel::Storage::uuid,
+        price => 0,
+        total => 0
+    });
 };
 
-__PACKAGE__->table('order_items');
-__PACKAGE__->autoupdate(1);
-__PACKAGE__->iterator_class('Handel::Iterator');
-__PACKAGE__->columns(All => qw(id orderid sku quantity price description total));
-__PACKAGE__->columns(Essential => qw(id orderid sku quantity price description total));
-__PACKAGE__->has_a(price => 'Handel::Currency');
-__PACKAGE__->has_a(total => 'Handel::Currency');
-__PACKAGE__->add_constraint('quantity', quantity => \&constraint_quantity);
-__PACKAGE__->add_constraint('price',    price    => \&constraint_price);
-__PACKAGE__->add_constraint('id',       id       => \&constraint_uuid);
-__PACKAGE__->add_constraint('orderid',  orderid  => \&constraint_uuid);
-__PACKAGE__->add_constraint('total',    total    => \&constraint_price);
-
 sub new {
-    my ($self, $data) = @_;
+    my ($class, $data) = @_;
 
     throw Handel::Exception::Argument( -details =>
         translate('Param 1 is not a HASH reference') . '.') unless
             ref($data) eq 'HASH';
 
-    if (!defined($data->{'id'}) || !constraint_uuid($data->{'id'})) {
-        $data->{'id'} = $self->uuid;
-    };
+    my $self = bless {
+        storage => $class->schema_instance->resultset($class->schema_source)->create($data)
+    }, $class;
 
-    return $self->construct($data);
+    return $self;
 };
 
 1;
@@ -42,7 +45,7 @@ __END__
 
 =head1 NAME
 
-Handel::Order::Item - Module representing an indivudal order line item
+Handel::Order::Item - Module representing an individual order line item
 
 =head1 SYNOPSIS
 
@@ -77,7 +80,7 @@ You can create a new C<Handel::Order::Item> object by calling the C<new> method:
 This is a lazy operation. No actual item record is created until the item object
 is passed into the C<add> method of a C<Handel::Order> object.
 
-=head1 METHOS
+=head1 METHODS
 
 =head2 description
 
